#include <config.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <xine.h>

#include <FL/Fl.H>
#include <FL/x.H>
#include <FL/fl_draw.H>
#include <FL/Enumerations.H>
#include <FL/filename.H>

#include "flxine.h"
#include "xine_iface_gui.h"
#include "gui_functions.h"
#include "xine_interface.h"


char main_dir[MAX_FLXINE_PATH + 256];
xine_interface *xinei = NULL;
int current_item = 0;

//------ FORWARD
static char main_mrl[MAX_FLXINE_PATH];
static char time_buffer[10];
static const char *version = PACKAGE_STRING;
static int stream_pos, stream_len;


// dvd:/ ==> OK
// dvd:/3 ==> OK
// dvd://dev/dvd ==> OK
// dvd://dev/dvd/3 ==> OK
// cdda:/ ==> OK
// cdda:/3 ==> OK
// file7.png ==> OK 
// /home/IMG/PNG4/file7.png ==> OK
// file:/file7.png ==> OK
// file://home/VIDEO/MPEG/DELTA.MPG ==> OK
// file:/`pwd`/file7.png ==> OK 
// file:/`pwd`/DELTA.MPG ==> OK 
// 

static void usage()
{
	puts("\n----------------------------------------------------------");
	puts("flxine\n");
	puts("flxine file_name\n");
	puts("flxine file:absolute_path/file_name\n");
	puts("flxine dvd:/[device_name][/title_number[.part_number]]\n");
	puts("flxine dvd:/DVD_image_file[/title_number[.part_number]]\n");
	puts("flxine dvd:/DVD_directory[/title_number[.part_number]]\n"); 
	puts("flxine vcd://[CD_image_or_device_name][@[letter]number]\n"); 
	puts("flxine vcdo://track_number\n"); 
	puts("flxine cdda:/[device][/track_number]\n");
	puts("cat FILE.xyz | flxine stdin:/\n"); 
	puts("--version\n");
	puts("--help\n");
	puts("----------------------------------------------------------");
}

static void clock_set_cb(void *v)
{
	if( (control_window->visible())  && (xinei->status() > 1) )
   {
	int stream_cpos, played_time, total_time;

	xinei->get_stream_time(&stream_cpos, &played_time, &total_time);

	played_time /= 1000;   //Time passed since stream start in ms
	total_time /= 1000;    //Total time of stream in ms

	sprintf(time_buffer,  "%02d:%02d:%02d",
	 played_time / 3600, (played_time % 3600) / 60,
	 played_time % 60);

	Clock->label(time_buffer);
   }
	Fl::repeat_timeout(1.0, clock_set_cb, v);
}

static void handle_completion()
{
	if(fl_xevent != NULL)
   { 
	XEvent e = *fl_xevent;

	xinei->send_vo_event(XINE_GUI_SEND_EXPOSE_EVENT, &e);
   }
}

static void ui_events(void *foo)
{
	int event_id;
	char ui_data[MAX_UI_DATA];

	*ui_data = 0; event_id = 0;

	if(xinei->get_ui_event(&event_id, ui_data))
   {
	if(event_id == XINE_EVENT_UI_SET_TITLE)//3
  {
	control_playing->align(FL_ALIGN_CENTER);
	control_playing->label(xinei->title());

	return;
  }

	if(event_id == XINE_EVENT_UI_PLAYBACK_FINISHED)//1
  {
	playlist_clicker();

	return;
  }

	if(event_id == XINE_EVENT_UI_MESSAGE)//4
  {
	printf("\tUI_MESSAGE(%s)\n",ui_data);

	return;
  }
// 2: XINE_EVENT_UI_CHANNELS_CHANGED
// 5: XINE_EVENT_FRAME_FORMAT_CHANGE
// 6: XINE_EVENT_AUDIO_LEVEL
// 7: XINE_EVENT_QUIT
// 8: XINE_EVENT_PROGRESS
// 9: XINE_EVENT_MRL_REFERENCE :deprecated
//10: XINE_EVENT_UI_NUM_BUTTONS
//11: XINE_EVENT_SPU_BUTTON 
//12: XINE_EVENT_DROPPED_FRAMES
//13: XINE_EVENT_MRL_REFERENCE_EXT
//14: XINE_EVENT_AUDIO_AMP_LEVEL
//15: XINE_EVENT_NBC_STATS

   }//if(xinei->get_ui_event
}

static void set_title()
{
    if(strcmp(xinei->mrl(), main_mrl) )
   {
	const char *cs, *title;

	title = xinei->title();
	if((cs = strrchr(title, '/'))) ++cs; else cs = title;

    strcpy(main_mrl,xinei->mrl());
    control_playing->label(cs);
   }
}

static void gui_update_omatic(void *foo)
{
	int stream_cpos, stream_time, stream_finish;	

	Fl::wait(0.5);

	xinei->set_video_attr(video_window->x(),
		video_window->y(),
		0,
		0,
		video_window->w(),  
		video_window->h() );

//If 'EXIT' button pressed without MRL:
//
	xinei->get_stream_time(&stream_cpos, &stream_time, &stream_finish);

	if( (control_window->visible()) && (xinei->status() > 1) )
   {
	if( (stream_pos+SLIDER_STEP < stream_cpos) 
	|| stream_len != stream_finish 
	)
  {	
	stream_pos = stream_cpos;

	if(stream_len != stream_finish)
 {
	set_title();
	stream_len = stream_finish;				
 }
	control_progress_bar->value(stream_pos);

	return;
  }
	else
	 set_title();

	control_progress_bar->value(stream_cpos);
   }
	if(stream_len == stream_finish
	|| xinei->program_stopped()

	  )
  {
	handle_completion();
  }

}//gui_update_omatic()

static void update_last_image(void *v)
{
	handle_completion();

	Fl::wait(0.01);
}

void playlist_clicker()
{
	if(current_item < gui_playlist_browser->size())
   {
	if(xinei->cd_running())
  {
	playlist_next();
  }
	return;
   }

	if(xinei->cd_running()) return;

	xinei->last_image_found(true);

	Fl::add_idle(update_last_image);
}

void Main_reset()
{
	Fl::remove_timeout(clock_set_cb, NULL);
    Fl::remove_idle(ui_events);
	Fl::remove_idle(gui_update_omatic);

	if(xinei->last_image_found())
   {
	xinei->last_image_found(false);;

	Fl::remove_idle(update_last_image);
   }
	stream_pos = stream_len = 0;

	control_progress_bar->value(0);
	control_playing->label("");

    Fl::add_idle(ui_events);
    Fl::add_idle(gui_update_omatic);
	Fl::add_timeout(1.0, clock_set_cb, NULL);
}

void Main_exit_program(void)
{
    gui_save_settings();

	Fl::remove_timeout(clock_set_cb, NULL);
    Fl::remove_idle(ui_events);
	Fl::remove_idle(gui_update_omatic);

	if(xinei->last_image_found())
   {
	xinei->last_image_found(false);;

	Fl::remove_idle(update_last_image);
   }

    xinei->~xine_interface();

	_exit(0);
}

int main(int argc, char *argv[])
{
	char *s;
	size_t len;
	int i;

	*main_mrl = 0;
	*main_dir = 0;
	current_item = 0;

	if(argc == 2)
   {
	s = argv[1];

	if(strcmp(s, "--help") == 0
	|| strcmp(s, "-help") == 0
	|| strcmp(s, "?") == 0)
  {
	usage();
	return 0;
  }
	if(strcmp(s, "--version") == 0)
  {
	printf("\n\t%s\n", version);
	return 0;
  }
   }//if(argc == 2)

	s = main_mrl;
 
	for(i = 1; i < argc; ++i)
   {
	if(i > 1)
  {
	len = strlen(s);
	strcpy(s + len, " ");
	s += len + 1;
  }
	strcpy(s, argv[i]);
   }

	stream_len = stream_pos = 0;

    XInitThreads();

	setenv("DVDCSS_CACHE","/tmp",1);

    xinei = new xine_interface();

	gui_init();	

	gui_load_plugins();

	video_window->show();
    control_window->show();	
	
	gui_load_settings();    
	
    xinei->set_parent(fl_xid(video_window),fl_display,fl_screen);

    xinei->init(gui_vid_driver_choice->text(),
		gui_audio_driver_choice->text());

	if(gui_vis_plugin_enable->value () 
	&& gui_vis_plugin_choice->text())
   {
	xinei->use_vis(gui_vis_plugin_enable->value (),
					gui_vis_plugin_choice->text ());
   }

    Fl::add_idle(ui_events);
	Fl::add_idle(gui_update_omatic);

	Fl::add_timeout(1.0, clock_set_cb, NULL);

	xinei->set_video_attr(video_window->x(),
		video_window->y(),
		0,
		0,
		video_window->w(),
		video_window->h() );

	if(*main_mrl)
   {
	if(strcmp(main_mrl, "cdda:/") == 0)
  {
	xinei->start_plugin(0);
  }
	else
	if(strcmp(main_mrl, "dvd:/") == 0)
  {
	xinei->start_plugin(3);
  }
	else
	if(strcmp(main_mrl, "dvb:/") == 0)
  {
	xinei->start_plugin(1);
  }
	else
	if(strcmp(main_mrl, "test:/") == 0)
  {
	xinei->start_plugin(4);
  }
	else
	if(strcmp(main_mrl, "vcd:/") == 0)
  {
	xinei->start_plugin(5);
  }
	else
  {
	if(strncmp(main_mrl, "file:", 5) == 0
	&& (main_mrl[5] != '/' && main_mrl[5] != '\\') )
 {
	strcpy(main_dir, main_mrl+5);
	strcpy(main_mrl, "file:");
	fl_filename_absolute(main_mrl+5, sizeof(main_mrl), main_dir);

	*main_dir = 0;
 }
	if( !xinei->open(main_mrl))
 {
	printf("%s:%d:\n\topen(%s)\n\tfailed.\n",__FILE__,__LINE__,main_mrl);
	*main_mrl = 0;
 }
	else
	if( !xinei->play())
 {
	printf("%s:%d:\n\tplay(%s)\n\tfailed.\n",__FILE__,__LINE__,main_mrl);

	*main_mrl = 0;
 }
  }
   }

    while(video_window->shown())
		Fl::wait();
    
	Main_exit_program();
	
  return 0;
}
