#include "xine_info.h"

//xine OSD definitions

#ifdef __cplusplus
extern "C"
{
#endif

#include <unistd.h> //sleep()

#define OVL_PALETTE_SIZE 256

	typedef struct clut_t
	{			/* CLUT == Color LookUp Table */
		uint8_t cb:8;
		uint8_t cr:8;
		uint8_t y:8;
		uint8_t foo:8;
	}
	__attribute__ ((packed)) clut_t;

	clut_t textpalettes_color[] = {
		/* white, no border, translucid */
		{0x00, 0x00, 0x00, 0x00},	//0
		{0x80, 0x80, 0x60, 0x00},	//1
		{0x80, 0x80, 0x70, 0x00},	//2
		{0x80, 0x80, 0x80, 0x00},	//3
		{0x80, 0x80, 0x80, 0x00},	//4
		{0x80, 0x80, 0x80, 0x00},	//5
		{0x80, 0x80, 0x80, 0x00},	//6
		{0x80, 0x80, 0xa0, 0x00},	//7
		{0x80, 0x80, 0xc0, 0x00},	//8
		{0x80, 0x80, 0xe0, 0x00},	//9
		{0x80, 0x80, 0xff, 0x00},	//10
		/* yellow, black border, transparent */
		{0x00, 0x00, 0x00, 0x00},	//0
		{0xe0, 0x80, 0x80, 0x00},	//1
		{0xc0, 0x80, 0x80, 0x00},	//2
		{0xa0, 0x80, 0x60, 0x00},	//3
		{0x80, 0x80, 0x40, 0x00},	//4
		{0x80, 0x80, 0x20, 0x00},	//5
		{0x80, 0x80, 0x00, 0x00},	//6
		{0x60, 0x84, 0x40, 0x00},	//7
		{0x40, 0x88, 0xd0, 0x00},	//8
		{0x00, 0x8a, 0xe0, 0x00},	//9
		{0x00, 0x90, 0xff, 0x00}	//10
	};

	static uint8_t textpalettes_trans[] = {
		/* white, no border, translucid */
		0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15,
		/* yellow, black border, transparent */
		0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15,
	};
#ifdef __cplusplus
}				//end extern "C"
#endif

struct xine_status_s
{
	const char *symbol;
	int status;
};

static struct xine_status_s xine_status[] =
{
	{
	"�", XINE_STATUS_IDLE} // 0
	,
	{
	"}", XINE_STATUS_STOP} // 1
	,
	{
	">", XINE_STATUS_PLAY} // 2
	,
	{
	"{", XINE_STATUS_QUIT} // 3
	,
	{
	NULL, 0}
};

struct xine_speeds_s
{
	const char *symbol;
	int speed;
};

static struct xine_speeds_s xine_speeds[] =
{
	{
	"<", XINE_SPEED_PAUSE}
	,
	{
	"@@>", XINE_SPEED_SLOW_4}
	,
	{
	"@>", XINE_SPEED_SLOW_2}
	,
	{
	">", XINE_SPEED_NORMAL}
	,
	{
	">$", XINE_SPEED_FAST_2}
	,
	{
	">$$$", XINE_SPEED_FAST_4}
	,
	{
	NULL, 0}
};

#define BAR_WIDTH 336
#define BAR_HEIGHT 25
#define OSD_BAR_PROGRESS 1
#define OSD_BAR_POS      2
#define OSD_BAR_STEPPER  3

const char *
_osd_get_speed_sym (int speed)
{
	int i;

	for (i = 0; xine_speeds[i].symbol != NULL; i++)
	{
		if(speed == xine_speeds[i].speed)
			return xine_speeds[i].symbol;
	}

	return NULL;
}

const char *
_osd_get_status_sym (int status)
{
	int i;

	for (i = 0; xine_status[i].symbol != NULL; i++)
	{
		if(status == xine_status[i].status)
			return xine_status[i].symbol;
	}

	return NULL;
}

// This thread is used to count down the osd display, 
// and when timed-out hide it
//
static void *osd_timer_thread(void *xine_gen)
{
	xine_info *xine_data = (xine_info *) xine_gen;

	while(!xine_data->osd_stop)
   {
	
//We only need to update once per second
//
	sleep(1);

	if(xine_data->osd_stop) break;

	pthread_mutex_lock(&xine_data->osd_mutex);

	if(xine_data->osd_info_visible)
  {
	xine_data->osd_info_visible--;

	if(!xine_data->osd_info_visible)
 {
	xine_osd_hide(xine_data->osd_info, 0);
 }
  }
	if(xine_data->osd_bar_visible)
  {
	xine_data->osd_bar_visible--;

	if(!xine_data->osd_bar_visible)
 {
	xine_osd_hide(xine_data->osd_bar[0], 0);
	xine_osd_hide(xine_data->osd_bar[1], 0);
 }
  }
	if(xine_data->osd_status_visible)
  {
	xine_data->osd_status_visible--;

	if(!xine_data->osd_status_visible)
 {
	xine_osd_hide(xine_data->osd_status, 0);
 }
  }
	pthread_mutex_unlock(&xine_data->osd_mutex);

   }
	xine_data->osd_stop = 0;
	return NULL;
}
