#include <config.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <FL/filename.H>
#include <FL/fl_utf8.h>

#include <xine.h>
#include <xine/xineutils.h> //homedir, thread

#include "flxine.h"
#include "xine_iface_gui.h"
#include "xine_info.h"
#include "gui_functions.h"
#include "osd_defs.h"
#include "xine_interface.h"

//#define DEBUG_START_PLUGIN
//#define DEBUG_INIT_INTERFACE

//---- define this of you want a triangle bar
//#define USE_OSD_BAR_STEPPER
//--------------------------------------------
static char POS_IN_STREAM_s[] = {"Position in Stream"};


static char config_file[2048];
static char ui_event_data[MAX_UI_DATA];

#ifdef INFORMATION_ONLY
/*------------------------------
#define XINE_STATUS_IDLE 0 :no mrl assigned
#define XINE_STATUS_STOP 1
#define XINE_STATUS_PLAY 2
#define XINE_STATUS_QUIT 3
-------------------------------*/
#endif //INFORMATION_ONLY


static void dest_size_cb(void *xine_gen, 
	int video_width, int video_height, double video_pixel_aspect,
	int *dest_width, int *dest_height, double *dest_pixel_aspect)
{
	int width, height, changed;
//
//--------- NEVER USED
//
	xine_info *xine_data = (xine_info *) xine_gen;
//If necessary, correct size with video_pixel_aspect
//
	width = xine_data->video_attr[4];
	height = xine_data->video_attr[5];
	changed = 0;

	if(video_pixel_aspect >= xine_data->aspect_ratio)
   {
	width  = (int)
	 (double)video_width * video_pixel_aspect / xine_data->aspect_ratio
		+ 0.5;
	xine_data->aspect_ratio = video_pixel_aspect;
	changed = 1;
   }
	else
   {
	height = (int)
	 (double)video_height * xine_data->aspect_ratio / video_pixel_aspect
		+ 0.5;
	xine_data->aspect_ratio = video_pixel_aspect;
	changed = 1;
   }

	if(changed)
   {
	*dest_width = width;
	*dest_height = height;
	*dest_pixel_aspect = xine_data->aspect_ratio;
   }
}

static void frame_output_cb(void *xine_gen, 
	int video_width, int video_height, double video_pixel_aspect,
	int *dest_x, int *dest_y,
	int *dest_width, int *dest_height, double *dest_pixel_aspect,
	int *win_x, int *win_y)
{
	xine_info *xine_data = (xine_info *) xine_gen;

	if(xine_data->test_is_running)
   {
	video_window->size(TEST_VIDEO_WIDTH, TEST_VIDEO_HEIGHT);
	canvas->size(TEST_VIDEO_WIDTH, TEST_VIDEO_HEIGHT);
   }
	else
	if(xine_data->cd_is_running)
   {
	video_window->size(CD_VIDEO_WIDTH, CD_VIDEO_HEIGHT);
	canvas->size(CD_VIDEO_WIDTH, CD_VIDEO_HEIGHT);
   }
	else
   {
	if(video_pixel_aspect >= xine_data->aspect_ratio)
  {
	video_width  = 
	  video_width * video_pixel_aspect / xine_data->aspect_ratio + .5;
  }
	else
  {
	video_height = 
	 video_height * xine_data->aspect_ratio / video_pixel_aspect + .5;
  }

	if(video_window->w() != canvas->w() 
	|| video_window->h() != canvas->h())
  {
	canvas->size(video_width, video_height);

	xine_data->video_attr[4] = video_width;
	xine_data->video_attr[5] = video_height;
  }
   }

	*win_x = xine_data->video_attr[0];
	*win_y = xine_data->video_attr[1];
	*dest_x = xine_data->video_attr[2];
	*dest_y = xine_data->video_attr[3];
	*dest_width = xine_data->video_attr[4];
	*dest_height = xine_data->video_attr[5];
	*dest_pixel_aspect = xine_data->aspect_ratio;
}

static void
xine_event_cb(void *xine_iface_gen, const xine_event_t * new_event)
{
	xine_interface *xine_iface = (xine_interface *) xine_iface_gen;

	if((new_event->type == XINE_EVENT_UI_PLAYBACK_FINISHED) //1
	|| (new_event->type == XINE_EVENT_UI_CHANNELS_CHANGED)//2
	|| (new_event->type == XINE_EVENT_UI_SET_TITLE) //3
	|| (new_event->type == XINE_EVENT_UI_MESSAGE)//4
	|| (new_event->type == XINE_EVENT_UI_NUM_BUTTONS)//10
	  )
	{
	if(new_event->data_length)
  {
	xine_ui_data_t *ui_data =
		(xine_ui_data_t*)new_event->data;

	xine_iface->set_ui_event(new_event->type, ui_data->str);
  }
		else
  {
	xine_iface->set_ui_event(new_event->type,
		"nothing here, please move along" );
  }
	}
}

void
xine_interface::send_simple_event(int event_type)
{
	xine_event_t event;

	event.stream = xine_data->xine_stream;
	event.data = NULL;
	event.data_length = 0;
	event.type = event_type;
	gettimeofday(&event.tv, NULL);

	xine_event_send(xine_data->xine_stream, &event);
}

bool xine_interface::program_stopped()
{
	return _program_stopped;
}

void xine_interface::last_image_found(bool v)
{
	_found_last_image = v;
}

bool xine_interface::last_image_found()
{
	return _found_last_image;
}

void xine_interface::cd_running(bool v)
{
    _cd_is_running = v;
	xine_data->cd_is_running = v;

	if(v == true) xine_data->test_is_running = false;//FIXME
	if(v == true) _dvd_is_running = false;
}

bool xine_interface::dvd_running()
{
	return _dvd_is_running;
}

bool xine_interface::cd_running()
{
    return _cd_is_running;
}

void xine_interface::test_running(bool v)
{
	xine_data->test_is_running = v;

	if(v == true) _cd_is_running = false;//FIXME
	if(v == true) _dvd_is_running = false;
}

bool xine_interface::test_running()
{
	return xine_data->test_is_running;
}

int xine_interface::send_mouse_event(int x, int y, int button)
{
	x11_rectangle_t rect;
	xine_input_data_t mouse_data;
	xine_event_t event;

	rect.x = x;
	rect.y = y;
	rect.w = rect.h = 0;
	int ret;

	ret =
	xine_port_send_gui_data(xine_data->xine_video,
	    XINE_GUI_SEND_TRANSLATE_GUI_TO_VIDEO,
	     (void *) &rect);
	
	if(ret == -1) return -1;

	mouse_data.x = rect.x;
	mouse_data.y = rect.y;

	event.stream = xine_data->xine_stream;

	if(button == 1)//FL_LEFT_MOUSE && FL_PUSH
   {
	mouse_data.button = 1;
	event.type = XINE_EVENT_INPUT_MOUSE_BUTTON;
   }
	else
   {
	mouse_data.button = 0;
	event.type = XINE_EVENT_INPUT_MOUSE_MOVE;
   }
	event.data = &mouse_data;
	event.data_length = sizeof(mouse_data);
	gettimeofday(&event.tv, NULL);

	xine_event_send(xine_data->xine_stream, &event);

	return 0;
}

xine_interface::xine_interface()
{
	xine_data = new xine_info;

	xine_data->xine = xine_new();

	snprintf(config_file, 2048, "%s/.flxine", xine_get_homedir());

	if( !fl_filename_isdir(config_file))
   {
	fl_mkdir(config_file, 0755);
   }
	snprintf(config_file, 2048, "%s/.flxine/config.flxine",
		xine_get_homedir() );
	
	xine_config_load(xine_data->xine, config_file);

	xine_init(xine_data->xine);

	const char *const *autoscan_plugins =
		xine_get_autoplay_input_plugin_ids(xine_data->xine);

	num_plugins = -1;
#ifdef DEBUG_INIT_INTERFACE
	fprintf (stderr, "---Autoscan plugins found:\n");
#endif

	for(int i = 0; autoscan_plugins[i] != NULL; ++i)
   {
#ifdef DEBUG_INIT_INTERFACE
		fprintf (stderr, "     %s -- %s\n",
			 autoscan_plugins[i],
			 xine_get_input_plugin_description (xine_data->xine,
							    autoscan_plugins
							    [i]));
#endif
	auto_plugins[i].plugin_id = autoscan_plugins[i];

	auto_plugins[i].plugin_desc =
		xine_get_input_plugin_description (xine_data->xine,
			autoscan_plugins[i]);

	num_plugins++;
   }

#ifdef DEBUG_INIT_INTERFACE
	const char *const *tmp_drivers;
	int i;
	tmp_drivers = xine_list_audio_output_plugins (xine_data->xine);

	fprintf (stderr, "---Audio drivers found:\n");
	for(i = 0; tmp_drivers[i] != NULL; i++)
		fprintf (stderr, "     %s\n", tmp_drivers[i]);

	tmp_drivers = xine_list_video_output_plugins (xine_data->xine);

	fprintf (stderr, "---Video drivers found:\n");
	for(i = 0; tmp_drivers[i] != NULL; i++)
		fprintf (stderr, "     %s\n", tmp_drivers[i]);

	tmp_drivers = xine_list_post_plugins (xine_data->xine);

	fprintf (stderr, "---Post plugins found:\n");
	for(i = 0; tmp_drivers[i] != NULL; i++)
		fprintf (stderr, "     %s\n", tmp_drivers[i]);
#endif //DEBUG_INIT_INTERFACE

	pthread_mutex_init(&xine_data->start_plugin_mutex, NULL);

	_initialized = false;
	_found_last_image = false;
	_cd_is_running = false;
	_program_stopped = false;

	pause_position = 0;

}//xine_interface::xine_interface()

xine_interface::~xine_interface()
{
	osd_deinit();
	deinit();

	xine_event_dispose_queue(xine_data->xine_event_queue);
	
	if(xine_data->vis_active)
   {
	xine_post_dispose(xine_data->xine, xine_data->post_plugin);
   }
	xine_exit(xine_data->xine);

	pthread_mutex_destroy(&xine_data->start_plugin_mutex);

	delete [] xine_data;
}

int
xine_interface::send_vo_event(int event_type, void *xine_event)
{
	if(event_type == XINE_GUI_SEND_EXPOSE_EVENT)//3
	{
		xine_port_send_gui_data(xine_data->xine_video, 
			event_type,
			(XEvent *) xine_event);
		return 1;
	}

	if(event_type == XINE_GUI_SEND_VIDEOWIN_VISIBLE)//5
	{
		int *is_visible = (int *) xine_event;

		xine_port_send_gui_data(xine_data->xine_video,
			event_type,
			is_visible);
		return 1;
	}

	if(event_type == XINE_GUI_SEND_DRAWABLE_CHANGED)//2
	{
		xine_port_send_gui_data(xine_data->xine_video,
			event_type, 
			xine_event);
		return 1;
	}
	return 0;

}

void
xine_interface::set_parent(Window parent_win, 
	Display * parent_disp, int parent_screen)
{
	double res_h, res_v;

	parent_window = parent_win;
	parent_display = parent_disp;
	parent_screen = parent_screen;

	res_h = (DisplayWidth (parent_display, parent_screen) * 1000
		 / DisplayWidthMM (parent_display, parent_screen));
	res_v = (DisplayHeight (parent_display, parent_screen) * 1000
		 / DisplayHeightMM (parent_display, parent_screen));

	xine_data->aspect_ratio = res_h / res_v;

	xine_data->vis.dest_size_cb = dest_size_cb;
	xine_data->vis.frame_output_cb = frame_output_cb;

	xine_data->vis.user_data = xine_data;

	xine_data->vis.d = parent_window;
	xine_data->vis.display = parent_display;
	xine_data->vis.screen = parent_screen;

}

void
xine_interface::set_video_attr(int win_x, int win_y, 
	int vid_x, int vid_y, int vid_w, int vid_h)
{
	xine_data->video_attr[0] = win_x;
	xine_data->video_attr[1] = win_y;
	xine_data->video_attr[2] = vid_x;
	xine_data->video_attr[3] = vid_y;
	xine_data->video_attr[4] = vid_w;
	xine_data->video_attr[5] = vid_h;
}

bool xine_interface::init(const char *vid_driver, const char *aud_driver)
{
	int video_type;

	xine_data->vis_active = 0;

	aud_driver = "alsa";//FIXME

// Initialize video and audio driver 
//
	xine_data->xine_audio =
		xine_open_audio_driver(xine_data->xine, aud_driver, NULL);

	xine_data->audio_driver = aud_driver;

	if(!vid_driver) vid_driver = "auto";

	if(!strcmp(vid_driver, "aa"))
		video_type = XINE_VISUAL_TYPE_AA;
	else 
	if(!strcmp(vid_driver, "fb"))
		video_type = XINE_VISUAL_TYPE_FB;
	else 
	if(!strcmp(vid_driver, "none"))
		video_type = XINE_VISUAL_TYPE_NONE;
	else
		video_type = XINE_VISUAL_TYPE_X11_2;

	xine_data->xine_video =
	 xine_open_video_driver(xine_data->xine, vid_driver,
		video_type, 
		(void *) &xine_data->vis);

	xine_data->video_driver = vid_driver;

	xine_data->xine_stream =
	 xine_stream_new(xine_data->xine, 
		xine_data->xine_audio,
		xine_data->xine_video);

// Listen and respond to any xine events that are received
//
	xine_data->xine_event_queue =
		xine_event_new_queue(xine_data->xine_stream);

	xine_event_create_listener_thread(xine_data->xine_event_queue,
		xine_event_cb, 
		this);

	xine_port_send_gui_data(xine_data->xine_video,
		XINE_GUI_SEND_DRAWABLE_CHANGED, 
		(void *)xine_data->vis.d);

	xine_port_send_gui_data(xine_data->xine_video,
		XINE_GUI_SEND_VIDEOWIN_VISIBLE, (void *) 1);

	events_pending = 0;

// Tell xine to de-interlace all video-out data 
//
	xine_set_param(xine_data->xine_stream, 
		XINE_PARAM_VO_DEINTERLACE,
		true);

	xine_data->osd_timeout = 3;
	xine_data->osd_enabled = 1;

	osd_init();

	if(xine_data->mrl) free(xine_data->mrl);
	xine_data->mrl = NULL;

	set_volume(gui_volume_knob->value());

	_initialized = true;

	return true;
}//xine_interface::init()

void xine_interface::deinit()
{
	xine_dispose(xine_data->xine_stream);
	xine_data->xine_stream = NULL;

	xine_close_audio_driver(xine_data->xine, xine_data->xine_audio);
	xine_data->xine_audio = NULL;

	xine_close_video_driver(xine_data->xine, xine_data->xine_video);
	xine_data->xine_video = NULL;
}

void
xine_interface::set_ui_event(int event_id, const char *event_data)
{
	strncpy(ui_event_data, event_data, MAX_UI_DATA);
	ui_event_type = event_id;

	events_pending = 1;
}

bool
xine_interface::get_ui_event(int *event_id, char *event_data)
{
	int ret = false;

	if(events_pending)
   {
	*event_id = ui_event_type;
	strncpy(event_data, ui_event_data, MAX_UI_DATA);

	ret = true;
	events_pending = 0;
   }
	return ret;
}

const char *const *
xine_interface::get_video_drivers()
{
	return xine_list_video_output_plugins (xine_data->xine);
}

const char *
xine_interface::get_current_video_driver()
{
	return xine_data->video_driver;
}

const char *const *
xine_interface::get_audio_drivers()
{
	return xine_list_audio_output_plugins (xine_data->xine);
}

const char *
xine_interface::get_current_audio_driver()
{
	return xine_data->audio_driver;
}

const char *const *
xine_interface::get_post_plugins(int type)
{
	return xine_list_post_plugins_typed (xine_data->xine, type);
}

void xine_interface::audio_language(int channel, char buf[])
{

    if(!xine_get_audio_lang(xine_data->xine_stream, channel, buf))
     sprintf(buf, "%3d", channel);

}

void xine_interface::open_cd_image()
{
	if(xine_data->mrl) free(xine_data->mrl);

	xine_data->mrl = strdup("cd-image/frame-300x200.png");

	if(xine_open(xine_data->xine_stream, xine_data->mrl))
   {
	Main_reset();

	play();
   }

}

//---------------- lang LANG --------------

void xine_interface::start_plugin(int id)
{
	const char * const *plugin_mrl;
	char *mrl;
	int ret;

	pthread_mutex_lock(&xine_data->start_plugin_mutex);

	plugin_mrl = get_plugin_autoplay_mrls(id);

	if(plugin_mrl == NULL) 
   {
	pthread_mutex_unlock(&xine_data->start_plugin_mutex);
	return;
   }
	mrl = strdup(plugin_mrl[0]); // HERE, NOT LATER

	stop();

	gui_playlist_browser->clear();

	playlist_add_plugin(id);

	current_item = 1;

	if(id == 3)
   {
	_dvd_is_running = true;
	xine_data->dvd_is_running = true;
	control_reset_language_and_subtitle();

	_cd_is_running = false;
	xine_data->cd_is_running = false;

	xine_data->test_is_running = false;
   }
	else
   {
	_cd_is_running = (id == 0);
	xine_data->cd_is_running = (id == 0);
	xine_data->test_is_running = (id == 4);

	_dvd_is_running = false;
	xine_data->dvd_is_running = false;
	control_hide_language_and_subtitle();
   }
	_found_last_image = false;
	_program_stopped = false;

	pause_position = 0;

	if(id == 0) // CD
   {
	open_cd_image(); 
   }

	ret = open(mrl);

	free(mrl);

	pthread_mutex_unlock(&xine_data->start_plugin_mutex);

	if(ret) play();
}

const char *
xine_interface::get_auto_plugin(int id)
{
	if((id > -1) && (id < num_plugins + 1))
   {
	return auto_plugins[id].plugin_id;
   }
	return "Invalid Entry";
}

const char *
xine_interface::get_auto_plugin_desc(int id)
{
	if((id > -1) && (id < num_plugins + 1))
   {
	return auto_plugins[id].plugin_desc;
   }
	return "Invalid Entry";
}

const char *const *
xine_interface::get_plugin_autoplay_mrls(int id)
{
	const char *const *autoplay_mrls;

	int num_mrls = 0;

	if((id > -1) && (id < num_plugins + 1))
   {
	autoplay_mrls =
		xine_get_autoplay_mrls(xine_data->xine,
					auto_plugins[id].plugin_id,
					&num_mrls);
	if(num_mrls > 0)
  {
#ifdef VERBOSE_INFO
	fprintf (stderr, "++++ There are %i mrls within %s\n",
		 num_mrls, autoplay_mrls[0]);
#endif
	return autoplay_mrls;
  }
   }
	return NULL;
}

char *
xine_interface::get_extensions ()
{
	return xine_get_file_extensions (xine_data->xine);
}

int
xine_interface::use_vis(bool enable, const char *vis_name)
{
	int was_playing, pos, time, length;

	if(status() == XINE_STATUS_PLAY)
	{
		was_playing = 1;
		get_stream_time(&pos, &time, &length);
	}

	if(enable
	&& ((strcmp(xine_data->vis_name, vis_name)) || !xine_data->vis_enabled))
	{
		stop();
#ifdef VERBOSE_INFO
		fprintf (stderr, "Old vis: %s New vis: %s\n",
			 xine_data->vis_name, vis_name);
#endif
		xine_data->vis_name = vis_name;
		xine_data->vis_enabled = 1;
		xine_data->vis_active = 0;

		if(was_playing)
		{
			play ();
			set_position (pos);
		}
	}
	else 
	if(xine_data->vis_enabled == 1 && !enable)
	{
		stop();
		xine_data->vis_enabled = 0;
		if(was_playing)
		{
			set_position (pos);
		}
	}

	return xine_data->vis_enabled;
}

const char *
xine_interface::get_vis()
{
	if(xine_data->vis_enabled)
		return xine_data->vis_name;

	return "none";
}

/***************************************************************************
** Media Playback and loading controls                                     **
***************************************************************************/

int
xine_interface::open(const char *mrl)
{
	int ret;

	if(xine_data->mrl) free(xine_data->mrl);

	xine_data->mrl = strdup(mrl);

	ret = xine_open(xine_data->xine_stream, mrl);

	if(ret) 
   {
	const char *cs = strrchr(mrl, '/');

	if(cs) ++cs; else cs = mrl;

	Main_reset();

	if(atoi(cs)) cs = title();

	control_playing->label(cs);

   }
	return ret;
}

bool xine_interface::start_stream(int position)
{
	bool ret;

	xine_post_out_t *audio_source =
		xine_get_audio_source(xine_data->xine_stream);

	if(xine_get_stream_info(xine_data->xine_stream,
				XINE_STREAM_INFO_HAS_VIDEO)
	&& xine_get_stream_info(xine_data->xine_stream,
				XINE_STREAM_INFO_VIDEO_HANDLED))
	{
		xine_post_wire_audio_port(audio_source,
				xine_data->xine_audio);

		xine_data->vis_active = 0;
	}
	else 
	if(xine_data->vis_enabled && !xine_data->vis_active)
	{
		xine_data->post_plugin = 
		xine_post_init(xine_data->xine,
				xine_data->vis_name, 0,
				&xine_data->xine_audio,
				&xine_data->xine_video);

		xine_post_wire_audio_port(audio_source,
				xine_data->post_plugin->audio_input[0]);

		xine_data->vis_active = 1;
	}
	else 
	if(!xine_data->vis_enabled && xine_data->vis_active)
	{
		xine_post_wire_audio_port(audio_source,
				xine_data->xine_audio);

		xine_data->vis_active = 0;
	}
	ret = false;

	if(xine_data->mrl)
   {
	ret =
	 xine_play(xine_data->xine_stream, position, 0);
   }
	return ret;

}//xine_interface::start_stream()

// Play or Pause
//
bool xine_interface::play()
{
	int stream_cpos, stream_time, stream_finish;

	if(status() != XINE_STATUS_PLAY)
   {
	return start_stream(pause_position);
   }
	get_stream_time(&stream_cpos, &stream_time, &stream_finish);

	pause_position = stream_cpos;

	if(get_speed() != XINE_SPEED_NORMAL)
	 set_speed(XINE_SPEED_NORMAL);
	else
	 set_speed(XINE_SPEED_PAUSE);

	osd_update_status();

	return 1;
}

void xine_interface::stop()
{
	if(xine_get_status(xine_data->xine_stream) != XINE_STATUS_PLAY)
	 return;

	xine_stop(xine_data->xine_stream);

	_program_stopped = true;

	osd_update_status();

	if(xine_data->vis_active)
   {		
	xine_post_out_t *audio_source =
		xine_get_audio_source (xine_data->xine_stream);

	xine_post_wire_audio_port(audio_source, xine_data->xine_audio);

	xine_post_dispose(xine_data->xine, xine_data->post_plugin);

	xine_data->vis_active=0;
   }

}

int xine_interface::eject()
{
	if(xine_get_status(xine_data->xine_stream) == XINE_STATUS_PLAY)
   {
	stop();
   }
	return xine_eject(xine_data->xine_stream);

}//xine_interface::eject()

void
xine_interface::faster()
{
	int current_speed = get_speed();

	if((current_speed > 0) && (current_speed < 16))
	{
		set_speed(current_speed * 2);
	}
	else 
	if(current_speed == 0)
	{
		set_speed(current_speed + 1);
	}
	osd_update_status();
}

void
xine_interface::slower()
{
	int current_speed = get_speed();

	if((current_speed > 1))
	{
		set_speed(current_speed / 2);
	}
	else 
	if(current_speed == 1)
	{
		set_speed(current_speed - 1);
	}
	osd_update_status();
}

void
xine_interface::set_speed(int new_speed)
{
	xine_set_param(xine_data->xine_stream, XINE_PARAM_SPEED, new_speed);
}

int
xine_interface::get_speed()
{
	return xine_get_param(xine_data->xine_stream, XINE_PARAM_SPEED);
}

int
xine_interface::set_position(int jump_position)
{
	if(xine_get_stream_info(xine_data->xine_stream, XINE_STREAM_INFO_SEEKABLE))
   {
	int speed;

	speed = get_speed();

	xine_play(xine_data->xine_stream, jump_position, 0);

	set_speed(speed);

    return 1;
   }
	return 0;
}

int
xine_interface::get_stream_time(int *stream_pos, int *position_time,
				 int *length_time)
{
/* int stream_pos     Current relative position in the stream (0-65535)
 * int position_time  Amount of time passed since stream start (millisecs)
 * int length_time    Length of stream in millisecs 
*/
	return xine_get_pos_length (xine_data->xine_stream, 
		stream_pos,
		position_time, 
		length_time);
}

/*****************************************************************************
* Begin volume control                                                       *
*****************************************************************************/
// Volume range is 0 .. 100
//
void xine_interface::set_volume(int volume)
{
	xine_set_param(xine_data->xine_stream, 
		XINE_PARAM_AUDIO_VOLUME,
		volume);
}

void xine_interface::toggle_mute(int v)
{
	xine_set_param(xine_data->xine_stream,
		XINE_PARAM_AUDIO_AMP_MUTE,
		v);
}
/******************************************************************************
* Begin navigation interfaces                                                 *
* these will be used for menu navigation by supported stream, such as a DVD   *
* menu, for instance.                                                         *
******************************************************************************/
bool
xine_interface::next()
{

	if(xine_get_stream_info(xine_data->xine_stream,
	    XINE_STREAM_INFO_HAS_CHAPTERS))
	{
		send_simple_event(XINE_EVENT_INPUT_NEXT);
		osd_update_status();
		return true;
	}
	return false;
}

bool
xine_interface::prev()
{
	if(xine_get_stream_info(xine_data->xine_stream,
	    XINE_STREAM_INFO_HAS_CHAPTERS))
	{
		send_simple_event(XINE_EVENT_INPUT_PREVIOUS);
		osd_update_status();
		return true;
	}
	return false;
}

void
xine_interface::nav_up()
{
	if(_cd_is_running) return;

	send_simple_event(XINE_EVENT_INPUT_UP);
}

void
xine_interface::nav_down()
{
	if(_cd_is_running) return;

	send_simple_event(XINE_EVENT_INPUT_DOWN);
}

void
xine_interface::nav_left()
{
	if(_cd_is_running) return;

	send_simple_event(XINE_EVENT_INPUT_LEFT);
}

void
xine_interface::nav_right()
{
	if(_cd_is_running) return;

	send_simple_event(XINE_EVENT_INPUT_RIGHT);
}

void
xine_interface::nav_select()
{
	if(_cd_is_running) return;

	send_simple_event(XINE_EVENT_INPUT_SELECT);
}

void
xine_interface::nav_menu1()
{
	if(_cd_is_running) return;

	send_simple_event(XINE_EVENT_INPUT_MENU1);
}

/*******************************************************************************
* Begin stream information                                                     *
* these functions can be called to retrieve information about the stream       *
*******************************************************************************/

int
xine_interface::status()
{
	return xine_get_status(xine_data->xine_stream);
}

/****************************************************************************
* Begin stream meta information                                             *
* these functions can be called to retrieve information within the          *
* stream such as MP3 ID3 tag data. 											*
*******************************************************************************/
const char *
xine_interface::mrl()
{
	return xine_data->mrl;
}

const char *
xine_interface::title()
{
	const char *title = xine_get_meta_info (xine_data->xine_stream,
						XINE_META_INFO_TITLE);

	if( !title) title = xine_data->mrl; return title;
}

/*******************************************************************************
* Begin Xine OSD functions                                                     *
* these functions can be called to display information over the video display  *
*******************************************************************************/

void
xine_interface::osd_init()
{
	int font_size = 17;

	xine_data->osd_info =
	 xine_osd_new(xine_data->xine_stream,
			0, 0, 900, (font_size * 6) + (5 * 3));

	xine_osd_set_font(xine_data->osd_info, "sans", font_size);
	xine_osd_set_text_palette(xine_data->osd_info,
				   XINE_TEXTPALETTE_WHITE_BLACK_TRANSPARENT,
				   XINE_OSD_TEXT1);

	xine_data->osd_bar[0] =
		xine_osd_new(xine_data->xine_stream,
			0, 0, BAR_WIDTH + 1, BAR_HEIGHT + 1);
	xine_osd_set_palette(xine_data->osd_bar[0], 
	 	(const uint32_t*)textpalettes_color, textpalettes_trans);

	xine_data->osd_bar[1] =
		xine_osd_new(xine_data->xine_stream,
			0, 0, BAR_WIDTH + 1, BAR_HEIGHT + 1);

	xine_osd_set_font(xine_data->osd_bar[1], "sans", font_size);
	xine_osd_set_text_palette (xine_data->osd_bar[1],
				   XINE_TEXTPALETTE_WHITE_BLACK_TRANSPARENT,
				   XINE_OSD_TEXT1);

	xine_data->osd_status =
		xine_osd_new(xine_data->xine_stream, 
			0, 0, 100, font_size);

	xine_osd_set_font (xine_data->osd_status, "cetus", font_size);
	xine_osd_set_text_palette (xine_data->osd_status,
				   XINE_TEXTPALETTE_WHITE_BLACK_TRANSPARENT,
				   XINE_OSD_TEXT1);

	xine_data->osd_initialized = true;
	xine_data->osd_stop = 0;
	pthread_mutex_init(&xine_data->osd_mutex, NULL);

	pthread_create(&xine_data->osd_timer_thread, NULL, 
		&osd_timer_thread,
		xine_data);
	
}//xine_interface::osd_init()

void xine_interface::osd_deinit()
{
	if(xine_data->osd_initialized == false) return;

	xine_data->osd_stop = 1;

	while(xine_data->osd_stop)
	 Fl::wait(0.5);

 	pthread_join(xine_data->osd_timer_thread, NULL);

	pthread_mutex_destroy(&xine_data->osd_mutex);

	if(xine_data->osd_info_visible)
   {
	xine_data->osd_info_visible = 0;
	xine_osd_hide(xine_data->osd_info, 0);
   }
	xine_osd_free(xine_data->osd_info);
	xine_data->osd_info = NULL;

	if(xine_data->osd_bar_visible)
   {
	xine_data->osd_bar_visible = 0;
	xine_osd_hide(xine_data->osd_bar[0], 0);
	xine_osd_hide(xine_data->osd_bar[1], 0);
   }
	xine_osd_free(xine_data->osd_bar[0]);
	xine_osd_free(xine_data->osd_bar[1]);

	xine_data->osd_bar[0] = NULL;
	xine_data->osd_bar[1] = NULL;

	if(xine_data->osd_status_visible)
   {
	xine_data->osd_status_visible = 0;
	xine_osd_hide(xine_data->osd_status, 0);
   }
	xine_osd_free(xine_data->osd_status);
	xine_data->osd_status = NULL;

}//xine_interface::osd_deinit()

#define MAX_BUF 256

void xine_interface::osd_stream_infos(void)
{

	if(xine_data->osd_enabled)
   {
	uint32_t vwidth, vheight, asrate;
	const char *vcodec, *acodec;
	char buffer[MAX_BUF], *p;
	int x, y;
	int w, h, osdw;
	int playedtime, totaltime, pos;
	int audiochannel, spuchannel, len;

	if(!xine_get_pos_length(xine_data->xine_stream, 
		&pos,
		&playedtime, 
		&totaltime))
	return;

	vcodec = xine_get_meta_info(xine_data->xine_stream,
			     XINE_META_INFO_VIDEOCODEC);
	acodec = xine_get_meta_info(xine_data->xine_stream,
			     XINE_META_INFO_AUDIOCODEC);
	vwidth = xine_get_stream_info(xine_data->xine_stream,
		         XINE_STREAM_INFO_VIDEO_WIDTH);
	vheight =
	 xine_get_stream_info(xine_data->xine_stream,
			     XINE_STREAM_INFO_VIDEO_HEIGHT);
	asrate = 
	 xine_get_stream_info(xine_data->xine_stream,
		         XINE_STREAM_INFO_AUDIO_SAMPLERATE);
	audiochannel =
	 xine_get_param(xine_data->xine_stream,
				 XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
	spuchannel =
	xine_get_param(xine_data->xine_stream,
				XINE_PARAM_SPU_CHANNEL);

	playedtime /= 1000;
	totaltime /= 1000;

	xine_osd_clear(xine_data->osd_info);

	if((vwidth == 0) && (vheight == 0))
  {
	vwidth = canvas->w();
	vheight = canvas->h();
	vcodec = xine_data->vis_name;

	if( !vcodec) vcodec = "unknown";
  }
	if(xine_data->cd_is_running) vcodec = NULL;

	y = x = 0;
	snprintf(buffer, MAX_BUF, "%s", xinei->title());

	xine_osd_get_text_size(xine_data->osd_info, buffer, &osdw, &h);

	p = buffer;

	while((unsigned int)osdw > (vwidth - 40))
  {
	*p = 0; ++p;
	*p = '.';
	*(p + 1) = '.';
	*(p + 2) = '.';
	xine_osd_get_text_size(xine_data->osd_info, p, &osdw, &h);
  }
	xine_osd_draw_text(xine_data->osd_info, x, y, p,
		    XINE_OSD_TEXT1);

	y += h;

	if(vcodec && vwidth && vheight)
  {
	snprintf(buffer, MAX_BUF, "%s: %dX%d", vcodec, 
		vwidth, vheight);

	xine_osd_draw_text(xine_data->osd_info, x, y, buffer,
					    XINE_OSD_TEXT1);
	xine_osd_get_text_size(xine_data->osd_info, buffer, &w, &h);

	if(w > osdw) osdw = w;

	y += h;
  }

	if(acodec && asrate)
  {
	snprintf(buffer, MAX_BUF, "%s: %dHz", acodec, asrate);

	xine_osd_draw_text(xine_data->osd_info, x, y, buffer,
			    XINE_OSD_TEXT1);

	xine_osd_get_text_size(xine_data->osd_info, buffer, &w, &h);

	if(w > osdw) osdw = w;

	y += h;
  }
	sprintf(buffer, "Audio: ");
	len = strlen (buffer);

	switch(audiochannel)
  {
	case -2:
		sprintf(buffer, "%soff", buffer);
		break;

	case -1:
		if(!xine_get_audio_lang(xine_data->xine_stream, audiochannel,
		     	&buffer[len]))
		 snprintf(buffer, MAX_BUF, "%sauto", buffer);
		break;

	default:
		if(!xine_get_audio_lang(xine_data->xine_stream, audiochannel,
		     	&buffer[len]))
		 sprintf(buffer, "%s%3d", buffer, audiochannel);
		break;
  }	//switch(

	sprintf(buffer, "%s, Spu: ", buffer);
	len = strlen (buffer);

	switch(spuchannel)
  {
	case -2:
		sprintf(buffer, "%soff", buffer);
		break;

	case -1:
		if(!xine_get_spu_lang(xine_data->xine_stream, spuchannel,
				&buffer[len]))
		 sprintf(buffer, "%sauto", buffer);
			break;

		default:
			if(!xine_get_spu_lang(xine_data->xine_stream, spuchannel,
				&buffer[len]))
		 sprintf(buffer, "%s%3d", buffer, spuchannel);
			break;
  }	//switch(

	sprintf(buffer, "%s.", buffer);
	xine_osd_draw_text(xine_data->osd_info, x, y, buffer,
				    XINE_OSD_TEXT1);

	xine_osd_get_text_size(xine_data->osd_info, buffer, &w, &h);

	if(w > osdw) osdw = w;

	y += h;

	if(totaltime > 0)
  {
	snprintf(buffer, MAX_BUF,
		 "%d:%.2d:%.2d (%.0f%%) of %d:%.2d:%.2d",
		 playedtime / 3600, (playedtime % 3600) / 60,
		 playedtime % 60,
		 ((float) playedtime / (float) totaltime) *
		 100, totaltime / 3600,
		 (totaltime % 3600) / 60, totaltime % 60);

	xine_osd_draw_text(xine_data->osd_info, x, y, buffer,
					    XINE_OSD_TEXT1);

	xine_osd_get_text_size(xine_data->osd_info, buffer, &w, &h);

	if(w > osdw) osdw = w;

	y += h;
  }
	osd_stream_position(pos);

	if((int)vwidth < osdw) vwidth = osdw;

	x = (vwidth - osdw)/2;

	xine_osd_set_position(xine_data->osd_info, x, 5);//15);

	pthread_mutex_lock(&xine_data->osd_mutex);

	xine_osd_show(xine_data->osd_info, 0);
	xine_data->osd_info_visible = xine_data->osd_timeout;

	pthread_mutex_unlock(&xine_data->osd_mutex);

   }//if(osd_enabled)

}//xine_interface::osd_stream_infos

void
xine_interface::osd_draw_bar(char *title, int min, int max, int val,
			      int type)
{

	if(xine_data->osd_enabled)
   {
	uint32_t vwidth, vheight;
	int bar_color[40];
	int i, x, y;
	float _val = (int) val;
	float _min = (int) min;
	float _max = (int) max;
	int pos;

	pthread_mutex_lock(&xine_data->osd_mutex); //FIXME

	if(max <= min)
		_max = (int) (min + 1);
	if(min >= max)
		_min = (int) (max - 1);
	if(val > max)
		_val = (int) max;
	if(val < min)
		_val = (int) min;

	pos = (int) (_val + -_min) / (int) ((_max + -_min) / 40);

	xine_osd_clear(xine_data->osd_bar[0]);
	xine_osd_clear(xine_data->osd_bar[1]);

	memset(&bar_color, (XINE_OSD_TEXT1 + 7), sizeof (int) * 40);

	switch(type)
  {
	case OSD_BAR_PROGRESS:
	case OSD_BAR_STEPPER:
		if(pos)
		 memset(bar_color, (XINE_OSD_TEXT1 + 21), pos * sizeof(int));
		break;

	case OSD_BAR_POS:
		if(pos)
		 bar_color[pos - 1] = (XINE_OSD_TEXT1 + 21);
		break;
  }	//switch(type)

	if((type == OSD_BAR_PROGRESS) || (type == OSD_BAR_POS))
  {
	x = 3;
	xine_osd_draw_rect(xine_data->osd_bar[0], x, 2,
			    x + 3, BAR_HEIGHT - 2,
			    XINE_OSD_TEXT1 + 9, 1);
	x += 8;

	for(i = 0; i < 40; i++, x += 8)
 {
	xine_osd_draw_rect(xine_data->osd_bar[0], x, 6,
			    x + 3, BAR_HEIGHT - 2,
			    bar_color[i], 1);
 }

	xine_osd_draw_rect(xine_data->osd_bar[0], x, 2,
			    x + 3, BAR_HEIGHT - 2,
			    XINE_OSD_TEXT1 + 9, 1);
 }
	else 
	if(type == OSD_BAR_STEPPER)
 {
	y = BAR_HEIGHT - 4;
	int step = y / 20;

	x = 3;

		for(i = 0; i < 37; i++, x += 8)
	   {
		xine_osd_draw_rect(xine_data->osd_bar[0], x, y,
			x + 3, BAR_HEIGHT - 2,
			bar_color[i], 1);

		if(!(i % 2))
			y -= step;
	   }
 }

	if(title)
 {
	int tw, th;

	xine_osd_get_text_size(xine_data->osd_bar[1], title, &tw, &th);

	xine_osd_draw_text(xine_data->osd_bar[1],
			    (BAR_WIDTH - tw) >> 1, 0, title, XINE_OSD_TEXT1);
 }
	vwidth = 
	 xine_get_stream_info(xine_data->xine_stream,
			       XINE_STREAM_INFO_VIDEO_WIDTH);
	vheight =
	 xine_get_stream_info(xine_data->xine_stream,
					      XINE_STREAM_INFO_VIDEO_HEIGHT);

	if(vwidth == 0) vwidth = canvas->w();
	if(vheight == 0) vheight = canvas->h();

	if(vwidth < BAR_WIDTH) vwidth = BAR_WIDTH;

	x = (vwidth - BAR_WIDTH)/2;
	y = vheight - BAR_HEIGHT - 40;

	xine_osd_set_position(xine_data->osd_bar[0], x, y);

	y -= BAR_HEIGHT;
	xine_osd_set_position(xine_data->osd_bar[1], x, y);

//	pthread_mutex_lock(&xine_data->osd_mutex); FIXME

	xine_osd_show(xine_data->osd_bar[0], 0);

	if(title)
	 xine_osd_show(xine_data->osd_bar[1], 0);

	xine_data->osd_bar_visible = xine_data->osd_timeout;

	pthread_mutex_unlock(&xine_data->osd_mutex);
   }
}//osd_draw_bar

void
xine_interface::osd_stream_position(int pos)
{
#ifdef USE_OSD_BAR_STEPPER
	osd_draw_bar(POS_IN_STREAM_s, 0, 65535, pos, OSD_BAR_STEPPER);
#else
	osd_draw_bar(POS_IN_STREAM_s, 0, 65535, pos, OSD_BAR_PROGRESS);
#endif
}

void
xine_interface::osd_update_status()
{

	if(xine_data->osd_enabled)
   {
	int status;
	char buffer[256];

	pthread_mutex_lock(&xine_data->osd_mutex);//FIXME

	status = xine_get_status(xine_data->xine_stream);

	xine_osd_clear(xine_data->osd_status);

/*
 * { : eject
 * [ : previous
 * | : | (thin)
 * @ : | (large)
 * ] : next
 * } : stop
 * $ : > (large)
 * > : play
 * < : pause
*/
	*buffer = 0;

	switch(status)
  {
	case XINE_STATUS_IDLE:// 0
	case XINE_STATUS_STOP:// 1
//		sprintf(buffer, "%s", (_osd_get_status_sym(status)));
		break;

	case XINE_STATUS_PLAY:// 2
	   {
		int speed = 
		 xine_get_param(xine_data->xine_stream,XINE_PARAM_SPEED);
		
		int secs;

		if(xine_get_pos_length(xine_data->xine_stream, NULL, &secs, NULL))
	  {
		secs /= 1000;
		sprintf(buffer, "%s %02d:%02d:%02d",
					 (_osd_get_speed_sym (speed)),
					 secs / (60 * 60), (secs / 60) % 60,
					 secs % 60);
	  }
		else
		 sprintf(buffer, "%s",
					 (_osd_get_speed_sym (speed)));
	   }
		break;

	case XINE_STATUS_QUIT://3
		/* noop */
		break;
  }	//switch(status)


	xine_osd_draw_text(xine_data->osd_status, 0, 0, buffer,
				    XINE_OSD_TEXT1);

	xine_osd_set_position(xine_data->osd_status, 20, 10);

//	pthread_mutex_lock(&xine_data->osd_mutex); FIXME

	xine_osd_show(xine_data->osd_status, 0);
	xine_data->osd_status_visible = xine_data->osd_timeout;

	pthread_mutex_unlock(&xine_data->osd_mutex);

   }//if(xine_data->osd_enabled)

}//xine_interface::osd_update_status()

void xine_interface::cycle_audio_channel(int channel)
{
	xine_set_param(xine_data->xine_stream, 
	 XINE_PARAM_AUDIO_CHANNEL_LOGICAL, channel);

	osd_update_status();
}

void xine_interface::cycle_spu_channel(int channel)
{
	xine_set_param(xine_data->xine_stream, 
	 XINE_PARAM_SPU_CHANNEL, channel);

	osd_update_status();
}

const char *
xine_interface::config_new_string(const char *key, const char *def_value,
				   const char *description, const char *help,
				   int exp_level, xine_config_cb_t changed_cb,
				   void *cb_data)
{
	return xine_config_register_string (xine_data->xine, key, def_value,
					    description, help, exp_level,
					    changed_cb, cb_data);
}

int
xine_interface::config_new_range(const char *key, int def_value,
				   int min, int max, const char *description,
				   const char *help, int exp_level,
				   xine_config_cb_t changed_cb, void *cb_data)
{
	return xine_config_register_range(xine_data->xine, key, def_value,
					   min, max, description, help,
					   exp_level, changed_cb, cb_data);
}

int
xine_interface::config_new_bool(const char *key, int def_value,
					const char *description,
				   const char *help, int exp_level,
				   xine_config_cb_t changed_cb, void *cb_data)
{
	return xine_config_register_bool(xine_data->xine, key, def_value,
					   description, help,
					   exp_level, changed_cb, cb_data);
}


int
xine_interface::config_lookup(const char *key, xine_cfg_entry_t * entry)
{
	return xine_config_lookup_entry(xine_data->xine, key, entry);
}

void
xine_interface::config_update(xine_cfg_entry_t * entry)
{
	xine_config_update_entry(xine_data->xine, entry);
}

void
xine_interface::config_save()
{
	xine_config_save(xine_data->xine, config_file);
}

