#ifndef _CLASS_XINE_INTERFACE_
#define _CLASS_XINE_INTERFACE_

#include <xine.h>

#include <FL/x.H>
#include <FL/Fl_Menu_Button.H>

#include "xine_info.h"

#define MAX_UI_DATA 256
#define MAX_PLUGIN_INFO 11

struct plugin_info
{
	const char *plugin_id;
	const char *plugin_desc;
};

class xine_interface
{
	private:
		Window parent_window;
		Display *parent_display;
		int parent_screen;

		xine_info *xine_data;
		int num_plugins;
		plugin_info auto_plugins[MAX_PLUGIN_INFO];

		bool _initialized;
		bool _found_last_image;
		bool _cd_is_running;
		bool _program_stopped;
		bool _dvd_is_running;

		int ui_event_type;
		int events_pending;
		int pause_position;

		void send_simple_event (int);

		void osd_init();
		void osd_deinit();
		bool start_stream(int);

    public:
		xine_interface();
		~xine_interface();

		int send_vo_event(int, void *);
		int send_mouse_event(int,int,int);
		void set_parent(Window, Display *, int);

		void set_video_attr(int, int, int, int, int, int);	
		bool init(const char *, const char *);
		void deinit();

		void last_image_found(bool);
		bool last_image_found();

		void cd_running(bool);
		bool cd_running();

		bool dvd_running();

		void test_running(bool v);
		bool test_running();

		bool program_stopped();

		void audio_language(int channel, char buf[]);
	//Events
		void set_ui_event(int, const char *);
		bool get_ui_event(int *, char *);

	//Plugin controls
		const char *const *get_video_drivers();
		const char *get_current_video_driver();

		const char *const *get_audio_drivers();
		const char *get_current_audio_driver();

		const char *const *get_post_plugins(int);
		void start_plugin(int);

		const char *get_auto_plugin(int);
		const char *get_auto_plugin_desc(int);
		const char *const *get_plugin_autoplay_mrls(int);

		char *get_extensions();
		int use_vis(bool, const char *);
		const char *get_vis();

	//Media loading and play control
		int open(const char *);
		bool play();
		void stop();

		int eject();
		void faster();
		void slower();

		void set_speed(int);
		int set_position(int);
		int get_speed();

		int get_stream_time(int *, int *, int *);
		void open_cd_image();

	//Volume Control
		void set_volume(int);
		void toggle_mute(int);

		int get_volume();
		int volume_up();
		int volume_down();
	//Navigation functions
		bool next();
		bool prev();
		void nav_up();

		void nav_down();
		void nav_left();
		void nav_right();

		void nav_select();
		void nav_menu1();

		void nav_menu2 ();
	//stream information
		int status();

		bool has_video();
		bool has_audio();
		uint32_t audio_bitrate();
		uint32_t audio_samplerate();

	//Current media information
		const char *mrl();
		const char *title();

		int track_number();
		const char *year();
		const char *artist();
		const char *album();
		const char *genre();
		const char *comment();

	//OSD Controls
		void osd_update();
		void osd_stream_infos();
		void osd_draw_bar(char *, int, int, int, int);

		void osd_stream_position(int);
		void osd_update_status();

	//Configuration Storage/Retrieval
		const char *config_new_string(const char *, 
			const char *, const char *,
			const char *, int, xine_config_cb_t,
			void *);
		int config_new_range(const char *, 
			int, int, int, const char *,
		    const char *, int, xine_config_cb_t, void *);
		
		int config_new_bool(const char *, int,
			const char *,
		    const char *,int, xine_config_cb_t, void *);
		
		int config_lookup(const char*, xine_cfg_entry_t *);
		void config_update(xine_cfg_entry_t *);

		void config_save();

		void cycle_audio_channel(int channel);
		void cycle_spu_channel(int channel);

		xine_stream_t *get_xine_stream()
	   { 
		if(xine_data) return xine_data->xine_stream; return NULL;
	   }
};


#endif
